#!/usr/bin/perl -w

# Copyright 2013, 2014, 2015, 2016, 2017, 2019, 2020 Kevin Ryde
#
# This file is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 3, or (at your option) any later
# version.
#
# This file is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
#
# You should have received a copy of the GNU General Public License along
# with this file.  See the file COPYING.  If not, see
# <http://www.gnu.org/licenses/>.

use 5.010;
use strict;
use Math::BigRat;
use Math::PlanePath::RationalsTree;

use FindBin;
use lib::abs $FindBin::Bin;
use MyPrintwrap ':all';
use MyRationals 'path_tree_to_hull_planar';

# uncomment this to run the ### lines
# use Smart::Comments;


my $max_depth = 14;

my $type = 'whole';
if (@ARGV) { $type = $ARGV[0]; }

print "% generated by tools/$FindBin::Script $type\n";

my $path = Math::PlanePath::RationalsTree->new;

sub H_by_planar {
  my ($depth, $type) = @_;
  my $planar = path_tree_to_hull_planar($path, $depth, $type);
  return $planar->area;
}

my @H_vector;
foreach my $depth (0 .. $max_depth) {
  push @H_vector, Math::BigRat->new(H_by_planar($depth,$type));
}
printwrap_indent("% GP-DEFINE  ");
printwrap_gp_vector
  (\@H_vector,
   $type eq 'whole' ? 'H_generated_samples' : 'h_generated_samples');

print "% end generated\n";
exit 0;
