#!/usr/bin/perl -w

# Copyright 2014, 2017, 2018 Kevin Ryde
#
# This file is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 3, or (at your option) any later
# version.
#
# This file is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
#
# You should have received a copy of the GNU General Public License along
# with this file.  If not, see <http://www.gnu.org/licenses/>.


# Print a tikz picture showing how the SB tree row regions copy out under
# left and right matrices, with dashed parallelograms in between.

use 5.010;
use strict;
use List::Util 'max','min';
use Math::Geometry::Planar;
use Math::Matrix;
use Math::PlanePath::RationalsTree;

use FindBin;
use lib $FindBin::Bin;
use MyPrintwrap ':all';
use MyRationals 'SB_left','SB_right','mvec_to_pq','mvec_to_str';


# Region printed is between depth and depth+1 copied out 3 times and which
# therefore is portions of depth+1 to depth+2.
#
my $depth = 3;


print "  % generated by tools/$FindBin::Script\n";

my $path = Math::PlanePath::RationalsTree->new(tree_type => 'SB');
my @vecs;
foreach my $n (($path->tree_depth_to_n($depth+1)
                .. $path->tree_depth_to_n_end($depth+1)),
               reverse($path->tree_depth_to_n($depth)
                       .. $path->tree_depth_to_n_end($depth))) {
  my ($x,$y) = $path->n_to_xy($n);
  push @vecs, Math::Matrix->new([$x],[$y]);
}

foreach my $m (SB_left(), SB_right()) {
  $Printwrap = 2;
  printwrap("\\draw[ultra thin] ");
  $Printwrap += 2;

  my $x_max = 0;
  my $y_max = 0;
  my @points;
  my %points;
  my $join = '';
  foreach my $i (0 .. $#vecs) {
    my $v = $vecs[$i];
    $v = $m*$v;

    my ($x,$y) = mvec_to_pq($v);
    my $p = [$x,$y];
    push @points, $p;
    $points{"$x,$y"} = 1;
    $x_max = max($x_max, $x);
    $y_max = max($y_max, $y);

    my $str = mvec_to_str($v);
    printwrap("$join($str)");
    $join = '--';
  }
  printwrap("--cycle;\n");

  my $planar = Math::Geometry::Planar->new;
  $planar->points(\@points);

  $Printwrap = 2;
  foreach my $x (0 .. $x_max) {
    foreach my $y (0 .. $y_max) {
      my $p = [$x,$y];
      next if $points{"$x,$y"};
      next unless $planar->isinside($p);
      printwrap("  \\fill ($x,$y) circle;");
    }
  }
}

printwrap_newline();
$Printwrap = 2;
printwrap("%\n");
my $d1 = $depth + 1;
my $d2 = $depth + 2;
my $d3 = $depth + 3;
printwrap("\\draw[ultra thin] ($d1,$d2)--($d2,$d1);\n");
printwrap("\\draw[ultra thin] ($d2,$d3)--($d3,$d2);\n");
printwrap("\\fill ($d2,$d2) circle;\n");

print "  % end generated\n";
exit 0;
