#!/usr/bin/perl -w

# Copyright 2014, 2015, 2017 Kevin Ryde
#
# This file is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 3, or (at your option) any later
# version.
#
# This file is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
#
# You should have received a copy of the GNU General Public License along
# with this file.  If not, see <http://www.gnu.org/licenses/>.


# Print tikz lines which are the rows of the SB tree.  These go clockwise
# around and show the region of p,q coverage.

use strict;
use Math::PlanePath::RationalsTree;

use FindBin;
use lib $FindBin::Bin;
use MyPrintwrap ':all';

my $path = Math::PlanePath::RationalsTree->new(tree_type => 'SB');

my $depth = 4;

print "  % generated by tools/$FindBin::Script\n";
print "  \\newcommand{\\MyDepth}{$depth}\n";

my ($n_lo, $n_hi) = $path->tree_depth_to_n_range($depth);
$Printwrap = 2;

# return $x,$y multiplied by 1+i
sub mul {
  my ($x,$y) = @_;
  # (x,y) + rot+90(x,y), is multiply (i+1)
  return ($x - $y,
          $y + $x);
}

my $x_total = 0;
my $y_total = 0;
my $count = 0;
foreach my $n ($n_lo .. $n_hi) {
  my ($x,$y) = $path->n_to_xy($n);
  ($x,$y) = mul($x,$y);
  my $name = ($n == $n_lo ? ' (start)'
              : $n == $n_hi ? ' (end)'
              : '');
  printwrap(" \\node$name at ($x,$y) [my dot];");
  $x_total += $x;
  $y_total += $y;
  $count++;
}
printwrap_newline();

my $gx = $x_total/$count;
my $gy = $y_total/$count;
print "  \\coordinate (G) at ($gx,$gy);\n";
print "  % end generated\n";
exit 0;
