#!/usr/bin/perl -w

# Copyright 2013, 2014, 2017, 2020 Kevin Ryde
#
# This file is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 3, or (at your option) any later
# version.
#
# This file is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
#
# You should have received a copy of the GNU General Public License along
# with this file.  If not, see <http://www.gnu.org/licenses/>.


# Print tikz trees for the six tree types.

use 5.004;
use strict;
use FindBin;
use Math::PlanePath::RationalsTree;
use Math::PlanePath::FractionsTree;

# tree types:
#     SB
#     CW
#     AYT
#     HCS
#     Bird
#     Drib
#     Kepler
#
my $tree_type = 'SB';
if (@ARGV) { $tree_type = $ARGV[0]; }

my $depth_limit = 2;
my $path;
if ($tree_type eq 'Kepler') {
  $path = Math::PlanePath::FractionsTree->new (tree_type => $tree_type);
} else {
  $path = Math::PlanePath::RationalsTree->new (tree_type => $tree_type);
}

print "  % generated by tools/$FindBin::Script $tree_type\n";

my $descend;
my @rows;
$descend = sub {
  my ($n) = @_;
  my $depth = $path->tree_n_to_depth($n);
  my ($x,$y) = $path->n_to_xy($n);
  my $indent = '  'x($depth+1);
  push @{$rows[$depth]}, "[$x;$y]";

  my $str = ($depth <= 4 ? "$x/$y" : "");
  print $indent;
  if ($depth == 0) { print '\\'; }
  print "node {$str}\n";
  return if $depth > $depth_limit;

  foreach my $n_child ($path->tree_n_children($n)) {
    print "${indent}child {\n";
    $descend->($n_child);
    print "${indent}}\n";
  }
};
$descend->($path->n_start);
print "  ;\n";
foreach my $depth (0 .. $#rows) {
  print "% GP-Test  ${tree_type}_row($depth) == [",
    join(',',@{$rows[$depth]}),
    "]\n";
}
print "  % end generated\n";

exit 0;
